@@ -4,6 +4,7 @@ from __future__ import print_function
 
 import argparse
 import functools
+import os
 import os.path
 import platform
 import shutil
@@ -15,11 +16,11 @@ from string import Template
 
 ################################################################
 
+DEBUG = "FOLDERIFY_DEBUG" in os.environ and os.environ["FOLDERIFY_DEBUG"] == "1"
 OLD_IMPLEMENTATION_FOLDER_TYPES = ["Yosemite", "pre-Yosemite"]
 
 def main():
 
-    DEFAULT_CACHE_DIR = os.path.expanduser("~/.folderify/cache")
     LOCAL_MACOS_VERSION = ".".join(platform.mac_ver()[0].split(".")[:2])
 
     parser = argparse.ArgumentParser(
@@ -77,41 +78,6 @@ Defaults to the version currently running (%s)." % LOCAL_MACOS_VERSION))
         help=("Don't trim margins from the mask. By default, transparent margins are trimmed from all 4 sides.")
     )
 
-    parser.add_argument(
-        "--cache", "-c",
-        action="store_true",
-        help="Cache the mask icon in the cache dir.")
-
-    parser.add_argument(
-        "--cache-dir",
-        type=str,
-        metavar="DIR",
-        default=DEFAULT_CACHE_DIR,
-        help="Use the specified cache directory (default: %s)." % DEFAULT_CACHE_DIR)
-
-    exclusive.add_argument(
-        "--cache-list",
-        action="store_true",
-        help="List all paths with cached masks.")
-
-    exclusive.add_argument(
-        "--cache-restore",
-        metavar="PATH",
-        type=str,
-        help="Restore folderified icon to the file/folder at PATH, \
-using the mask image in the cache for that path.")
-
-    exclusive.add_argument(
-        "--cache-restore-all",
-        action="store_true",
-        help="Restore all paths that have been cached.")
-
-    exclusive.add_argument(
-        "--cache-remove",
-        metavar="PATH",
-        type=str,
-        help="Remove the cached mask for the file/folder at PATH.")
-
     parser.add_argument(
         "--verbose", "-v",
         action="store_true",
@@ -119,33 +85,14 @@ using the mask image in the cache for that path.")
 
     ################################################################
 
-    def cache_path_for_target(target):
-        return args.cache_dir + os.path.abspath(target) + ".mask"
-
-    ################################################################
-
     args = parser.parse_args()
 
-    if args.cache and not (args.mask and args.target):
-        parser.error("Must specify mask and target in order to use --cache.")
-
     if args.mask and not os.path.exists(args.mask):
         parser.error("Mask file does not exist: %s" % args.mask)
 
     if args.target and not os.path.exists(args.target):
         parser.error("Target file/folder does not exist: %s" % args.target)
 
-    if args.cache_restore and not os.path.exists(args.cache_restore):
-        parser.error(
-            "File/folder does not exist (so the icon cannot be restored): %s" % args.cache_restore)
-    if args.cache_restore and not os.path.exists(cache_path_for_target(args.cache_restore)):
-        parser.error(
-            "File/folder is not in cache (so the icon cannot be restored): %s" % args.cache_restore)
-
-    if args.cache_remove and not os.path.exists(cache_path_for_target(args.cache_remove)):
-        parser.error(
-            "File/folder is not in cache (and cannot be removed): %s" % args.cache_remove)
-
     ################################################################
 
     data_folder = os.path.dirname(sys.modules[__name__].__file__)
@@ -167,8 +114,6 @@ using the mask image in the cache for that path.")
     iconutil_path = "iconutil"
     seticon_path = os.path.join(data_folder, "lib", "seticon")
 
-    cached_mask_suffix = ".mask"
-
     ################################################################
 
     # There are clever ways to do recursive flattening, but this works just fine.
@@ -192,7 +137,8 @@ using the mask image in the cache for that path.")
 
         global processes
 
-        name, icon_size, dims, b, w = params
+        name, icon_size, centering, dims, b, w = params
+        centering_width, centering_height = centering
         width, height, offset_center = dims
         black_blur, black_offset = b
         white_blur, white_offset, white_opacity = w
@@ -202,14 +148,24 @@ using the mask image in the cache for that path.")
 
         SIZED_MASK = os.path.join(temp_folder, "%s_1.0_SIZED_MASK.png" % name)
         try:
+            print()
             subprocess.check_call(p(
                 convert_path,
-                "-background", "transparent",
-                g(mask,
+                g(
+                    # We first center the image in the max size, to avoid pixel
+                    # rounding when the trimmed image is an odd width. (The extra
+                    # pixel would extend to the right, making the right margin
+                    # 1px slimmer. This is quite noticable on 16x16.)
+                    mask,
+                    "-background", "transparent",
                     [] if args.no_trim else "-trim",
-                    "-resize", ("%dx%d" % (width, height)),
+                    "-resize", ("%dx%d" % (centering_width, centering_height)),
                     "-gravity", "Center",
+                    "-extent", ("%dx%d" % (centering_width, centering_height))
                 ),
+                "-background", "transparent",
+                "-resize", ("%dx%d" % (width , height)),
+                "-gravity", "Center",
                 "-extent", ("%dx%d+0-%d" % (icon_size, icon_size, offset_center)),
                 SIZED_MASK
             ))
@@ -228,30 +184,38 @@ or
         FILE_OUT = os.path.join(iconset_folder, "icon_%s.png" % name)
         template_icon = os.path.join(template_folder, "icon_%s.png" % name)
 
+        def process(step_name, args):
+            if DEBUG:
+                file_name = os.path.join(temp_folder, "%s_%s.png" % (name, step_name))
+                print(file_name)
+                subprocess.Popen(p(convert_path, args, file_name)).wait()
+                return file_name
+            else:
+                return args
+
         def colorize(step_name, fill, input):
-            return g(input, "-fill", fill, "-colorize", "100, 100, 100")
+            return process(step_name, g(input, "-fill", fill, "-colorize", "100, 100, 100"))
         
         def opacity(step_name, fraction, input):
-            return g(input, "-channel", "Alpha", "-evaluate", "multiply", fraction)
+            return process(step_name, g(input, "-channel", "Alpha", "-evaluate", "multiply", fraction))
         
         def blur_down(step_name, blur_px, offset_px, input):
-            return g(input, "-motion-blur", ("0x%d-90" % blur_px),
-                            "-page", ("+0+%d" % offset_px), "-background", "none", "-flatten")
+            return process(step_name, g(input, "-motion-blur", ("0x%d-90" % blur_px), "-page", ("+0+%d" % offset_px), "-background", "none", "-flatten"))
 
         def mask_down(step_name, mask_operation, input, mask):
-            return g(input, mask, "-alpha", "Set", "-compose", mask_operation, "-composite")
+            return process(step_name, g(input, mask, "-alpha", "Set", "-compose", mask_operation, "-composite"))
         
         def negate(step_name, input):
-            return g(input, "-negate")
+            return process(step_name, g(input, "-negate"))
 
-        FILL_COLORIZED = colorize("1.1_FILL_COLORIZED", "rgb(8, 134, 206)", SIZED_MASK)
-        FILL = opacity("1.2_FILL", "0.5", FILL_COLORIZED)
+        FILL_COLORIZED = colorize("2.1_FILL_COLORIZED", "rgb(8, 134, 206)", SIZED_MASK)
+        FILL = opacity("2.2_FILL", "0.5", FILL_COLORIZED)
 
-        BLACK_NEGATED = negate("2.1_BLACK_NEGATED", SIZED_MASK)
-        BLACK_COLORIZED = colorize("2.2_BLACK_COLORIZED", "rgb(58, 152, 208)", BLACK_NEGATED)
-        BLACK_BLURRED = blur_down("2.3_BLACK_BLURRED", black_blur, black_offset, BLACK_COLORIZED)
-        BLACK_MASKED = mask_down("2.4_BLACK_MASKED", "Dst_In", BLACK_BLURRED, SIZED_MASK)
-        BLACK_SHADOW = opacity("2.5_BLACK_SHADOW", "0.5", BLACK_MASKED)
+        BLACK_NEGATED = negate("3.1_BLACK_NEGATED", SIZED_MASK)
+        BLACK_COLORIZED = colorize("3.2_BLACK_COLORIZED", "rgb(58, 152, 208)", BLACK_NEGATED)
+        BLACK_BLURRED = blur_down("3.3_BLACK_BLURRED", black_blur, black_offset, BLACK_COLORIZED)
+        BLACK_MASKED = mask_down("3.4_BLACK_MASKED", "Dst_In", BLACK_BLURRED, SIZED_MASK)
+        BLACK_SHADOW = opacity("3.5_BLACK_SHADOW", "0.5", BLACK_MASKED)
 
         WHITE_COLORIZED = colorize("4.1_WHITE_COLORIZED", "rgb(174, 225, 253)", SIZED_MASK)
         WHITE_BLURRED = blur_down("4.2_WHITE_BLURRED", white_blur, white_offset, WHITE_COLORIZED)
@@ -360,9 +324,15 @@ or
 
     ################################################################
 
-    def create_and_set_icns(mask, target=None, add_to_cache=False, is_from_cache=False):
+    def create_and_set_icns(mask, target=None):
 
-        temp_folder = tempfile.mkdtemp()
+        if DEBUG:
+            temp_folder = "%s-folderify-debug" % mask
+            if not os.path.exists(temp_folder):
+                os.mkdir(temp_folder)
+            subprocess.Popen(["open", temp_folder]).wait()
+        else:
+            temp_folder = tempfile.mkdtemp()
 
         if target:
             iconset_folder = os.path.join(temp_folder, "iconset.iconset")
@@ -371,10 +341,6 @@ or
             os.mkdir(iconset_folder)
 
             print_prefix = target
-            if is_from_cache:
-                print("[%s] Restoring from cache." % (print_prefix))
-            else:
-                print("[%s] <= [%s]" % (print_prefix, mask))
         else:
             root, _ = os.path.splitext(mask)
             iconset_folder = root + ".iconset"
@@ -388,15 +354,6 @@ or
             print("[%s] => [%s]" % (print_prefix, iconset_folder))
             print("[%s] => [%s]" % (print_prefix, icns_file))
 
-        if (add_to_cache):
-            mask_cache_path = cache_path_for_target(target)
-            mask_cache_folder = os.path.dirname(mask_cache_path)
-            if not os.path.exists(mask_cache_folder):
-                os.makedirs(mask_cache_folder)
-            shutil.copyfile(mask, mask_cache_path)
-            print("[%s] Storing in cache => [%s]" %
-                  (print_prefix, mask_cache_path))
-
         # The following can be excluded, since they are essentially
         # indistinguishable from the preceding @2x resolution:
         #
@@ -410,16 +367,16 @@ or
         # Data: Name, icon size, dimensions, black shadow, white top shadow, white bottom shadow
         inputs = {
             "BigSur": [
-                ["16x16",      16, (12, 6, 1), (0, 2), (2, 0, "0.5")],
-                ["16x16@2x",   32, (24, 12, 1), (0, 2), (2, 1, "0.35")],
-                ["32x32",      32, (24, 12, 1), (0, 2), (2, 1, "0.35")], # Can be excluded
-                ["32x32@2x",   64, (48, 24, 3), (0, 2), (2, 1, "0.6")],
-                ["128x128",    128, (96, 48, 6), (0, 2), (2, 1, "0.6")],
-                ["128x128@2x", 256, (192, 96, 12), (0, 2), (2, 1, "0.6")], # Can be excluded
-                ["256x256",    256, (192, 96, 12), (0, 2), (2, 1, "0.6")],
-                ["256x256@2x", 512, (384, 192, 24), (0, 2), (2, 1, "0.75")], # Can be excluded
-                ["512x512",    512, (384, 192, 24), (0, 2), (2, 1, "0.75")],
-                ["512x512@2x", 1024, (768, 384, 48), (0, 2), (2, 1, "0.75")]
+                ["16x16",      16, (768, 384), (12, 6, 2), (0, 2), (2, 0, "0.5")],
+                ["16x16@2x",   32, (768, 384), (24, 12, 2), (0, 2), (2, 1, "0.35")],
+                ["32x32",      32, (768, 384), (24, 12, 2), (0, 2), (2, 1, "0.35")], # Can be excluded
+                ["32x32@2x",   64, (768, 384), (48, 24, 3), (0, 2), (2, 1, "0.6")],
+                ["128x128",    128, (768, 384), (96, 48, 6), (0, 2), (2, 1, "0.6")],
+                ["128x128@2x", 256, (768, 384), (192, 96, 12), (0, 2), (2, 1, "0.6")], # Can be excluded
+                ["256x256",    256, (768, 384), (192, 96, 12), (0, 2), (2, 1, "0.6")],
+                ["256x256@2x", 512, (768, 384), (384, 192, 24), (0, 2), (2, 1, "0.75")], # Can be excluded
+                ["512x512",    512, (768, 384), (384, 192, 24), (0, 2), (2, 1, "0.75")],
+                ["512x512@2x", 1024, (768, 384), (768, 384, 48), (0, 2), (2, 1, "0.75")]
             ],
             "Yosemite": [
                 ["16x16",       12,   8,  1], ["16x16@2x",    26,  14,  2],
@@ -473,7 +430,8 @@ or
         ])
 
         # Clean up.
-        shutil.rmtree(temp_folder)
+        if not DEBUG:
+            shutil.rmtree(temp_folder)
 
         # Reveal target.
         if args.reveal:
@@ -487,60 +445,15 @@ or
         if args.verbose:
             print("[%s] Done." % (print_prefix))
 
-    def target_for_cache_path(cache_path):
-        assert(cache_path.endswith(cached_mask_suffix))
-        intermediate = cache_path[:-len(cached_mask_suffix)]
-        return os.path.join("/", os.path.relpath(intermediate, args.cache_dir))
-
-    def restore_from_cache(target):
-        mask_cache_path = cache_path_for_target(target)
-        if args.verbose:
-            print("[%s] Mask path from cache: %s" % (target, mask_cache_path))
-        create_and_set_icns(
-            mask_cache_path,
-            target=target,
-            add_to_cache=False,
-            is_from_cache=True
-        )
-
-    def cached_targets():
-        for folder, _, files in os.walk(args.cache_dir):
-            for f in files:
-                if f.endswith(cached_mask_suffix):
-                    cache_path = os.path.join(folder, f)
-                    yield target_for_cache_path(cache_path)
-
     if args.mask:
-        if args.cache:
-            assert(args.target)
         create_and_set_icns(
             args.mask,
-            target=args.target,
-            add_to_cache=args.cache,
-            is_from_cache=False
+            target=args.target
         )
 
-    elif args.cache_restore:
-        restore_from_cache(args.cache_restore)
-
-    elif args.cache_remove:
-        os.remove(cache_path_for_target(args.cache_remove))
-
-    elif args.cache_restore_all:
-        for target in cached_targets():
-            if os.path.exists(target):
-                restore_from_cache(target)
-            else:
-                print("[%s] Target no longer exists. Skipping." % target)
-
-    elif args.cache_list:
-        for target in cached_targets():
-            print(target)
-
     else:
         parser.print_help()
 
-
 __main__ = main
 
 if __name__ == "__main__":
